@@ -48,7 +48,6 @@ The parts not included are excluded by #ifndef UNIV_INNOCHECKSUM. */
 #include "buf0buf.h"             /* buf_page_is_corrupted */
 #include "page0zip.h"            /* page_zip_*() */
 #include "trx0undo.h"            /* TRX_* */
-#include "ut0crc32.h"            /* ut_crc32_init() */
 #include "fil0crypt.h"           /* fil_space_verify_crypt_checksum */
 
 #include <string.h>
@@ -635,7 +634,7 @@ static bool update_checksum(byte* page, uint32_t flags)
 	} else if (use_full_crc32) {
 		ulint payload = buf_page_full_crc32_size(page, NULL, NULL)
 			- FIL_PAGE_FCRC32_CHECKSUM;
-		checksum = ut_crc32(page, payload);
+		checksum = my_crc32c(0, page, payload);
 		byte* c = page + payload;
 		if (mach_read_from_4(c) == checksum) return false;
 		mach_write_to_4(c, checksum);
@@ -1128,11 +1127,15 @@ print_summary(
 	fprintf(fil_out, "index_id\t#pages\t\t#leaf_pages\t#recs_per_page"
 		"\t#bytes_per_page\n");
 
-	for (std::map<unsigned long long, per_index_stats>::const_iterator it = index_ids.begin();
-	     it != index_ids.end(); it++) {
-		const per_index_stats& index = it->second;
+	for (const auto &ids : index_ids) {
+		const per_index_stats& index = ids.second;
+		if (!index.pages) {
+			DBUG_ASSERT(index.free_pages);
+			continue;
+		}
+
 		fprintf(fil_out, "%lld\t\t%lld\t\t%lld\t\t%lld\t\t%lld\n",
-			it->first, index.pages, index.leaf_pages,
+			ids.first, index.pages, index.leaf_pages,
 			index.total_n_recs / index.pages,
 			index.total_data_bytes / index.pages);
 	}
