@@ -43,7 +43,7 @@ C_MODE_START
 extern unsigned int mysql_server_last_errno;
 extern char mysql_server_last_error[MYSQL_ERRMSG_SIZE];
 static my_bool emb_read_query_result(MYSQL *mysql);
-static void emb_free_embedded_thd(MYSQL *mysql);
+static void free_embedded_thd(MYSQL *mysql);
 static bool embedded_print_errors= 0;
 
 extern "C" void unireg_clear(int exit_code)
@@ -121,7 +121,7 @@ emb_advanced_command(MYSQL *mysql, enum enum_server_command command,
       thd->killed= NOT_KILLED;
     else
     {
-      emb_free_embedded_thd(mysql);
+      free_embedded_thd(mysql);
       thd= 0;
     }
   }
@@ -430,7 +430,7 @@ int emb_unbuffered_fetch(MYSQL *mysql, char **row)
   return 0;
 }
 
-static void emb_free_embedded_thd(MYSQL *mysql)
+static void free_embedded_thd(MYSQL *mysql)
 {
   THD *thd= (THD*)mysql->thd;
   server_threads.erase(thd);
@@ -453,12 +453,23 @@ static MYSQL_RES * emb_store_result(MYSQL *mysql)
   return mysql_store_result(mysql);
 }
 
-int emb_read_change_user_result(MYSQL *mysql)
+static int emb_read_change_user_result(MYSQL *mysql)
 {
   mysql->net.read_pos= (uchar*)""; // fake an OK packet
   return mysql_errno(mysql) ? (int)packet_error : 1 /* length of the OK packet */;
 }
 
+static void emb_on_close_free(MYSQL *mysql)
+{
+  my_free(mysql->info_buffer);
+  mysql->info_buffer= 0;
+  if (mysql->thd)
+  {
+    free_embedded_thd(mysql);
+    mysql->thd= 0;
+  }
+}
+
 MYSQL_METHODS embedded_methods= 
 {
   emb_read_query_result,
@@ -468,12 +479,12 @@ MYSQL_METHODS embedded_methods=
   emb_fetch_lengths, 
   emb_flush_use_result,
   emb_read_change_user_result,
+  emb_on_close_free,
   emb_list_fields,
   emb_read_prepare_result,
   emb_stmt_execute,
   emb_read_binary_rows,
   emb_unbuffered_fetch,
-  emb_free_embedded_thd,
   emb_read_statistics,
   emb_read_query_result,
   emb_read_rows_from_cursor
@@ -485,7 +496,7 @@ MYSQL_METHODS embedded_methods=
 
 char **copy_arguments(int argc, char **argv)
 {
-  uint length= 0;
+  size_t length= 0;
   char **from, **res, **end= argv+argc;
 
   for (from=argv ; from != end ; from++)
@@ -515,7 +526,6 @@ int init_embedded_server(int argc, char **argv, char **groups)
    */
   int *argcp;
   char ***argvp;
-  int fake_argc = 1;
   char *fake_argv[] = { (char *)"", 0 };
   const char *fake_groups[] = { "server", "embedded", 0 };
   my_bool acl_error;
@@ -528,16 +538,14 @@ int init_embedded_server(int argc, char **argv, char **groups)
   if (init_early_variables())
     return 1;
 
-  if (argc)
+  if (!argc)
   {
-    argcp= &argc;
-    argvp= (char***) &argv;
-  }
-  else
-  {
-    argcp= &fake_argc;
-    argvp= (char ***) &fake_argv;
+    argc= 1;
+    argv= fake_argv;
   }
+  argcp= &argc;
+  argvp= &argv;
+
   if (!groups)
     groups= (char**) fake_groups;
 
@@ -567,6 +575,8 @@ int init_embedded_server(int argc, char **argv, char **groups)
   if (ho_error != 0)
     return 1;
 
+  my_timer_init(&sys_timer_info);
+
   if (init_common_variables())
   {
     mysql_server_end();
@@ -1100,11 +1110,11 @@ bool Protocol_text::store_field_metadata(const THD * thd,
   client_field->flags= (uint16) server_field.flags;
   client_field->decimals= server_field.decimals;
 
-  client_field->db_length=		strlen(client_field->db);
-  client_field->table_length=		strlen(client_field->table);
-  client_field->name_length=		strlen(client_field->name);
-  client_field->org_name_length=	strlen(client_field->org_name);
-  client_field->org_table_length=	strlen(client_field->org_table);
+  client_field->db_length=		(uint)strlen(client_field->db);
+  client_field->table_length=		(uint)strlen(client_field->table);
+  client_field->name_length=		(uint)strlen(client_field->name);
+  client_field->org_name_length=	(uint)strlen(client_field->org_name);
+  client_field->org_table_length=	(uint)strlen(client_field->org_table);
 
   client_field->catalog= dup_str_aux(field_alloc, "def", 3, cs, thd_cs);
   client_field->catalog_length= 3;
@@ -1378,12 +1388,12 @@ bool Protocol::net_store_data(const uchar *from, size_t length)
 
   if (!(field_buf= (char*) alloc_root(alloc, length + sizeof(uint) + 1)))
     return TRUE;
-  *(uint *)field_buf= length;
+  *(uint *)field_buf= (uint)length;
   *next_field= field_buf + sizeof(uint);
   memcpy((uchar*) *next_field, from, length);
   (*next_field)[length]= 0;
   if (next_mysql_field->max_length < length)
-    next_mysql_field->max_length=length;
+    next_mysql_field->max_length=(ulong)length;
   ++next_field;
   ++next_mysql_field;
   return FALSE;
@@ -1393,7 +1403,7 @@ bool Protocol::net_store_data(const uchar *from, size_t length)
 bool Protocol::net_store_data_cs(const uchar *from, size_t length,
                               CHARSET_INFO *from_cs, CHARSET_INFO *to_cs)
 {
-  uint conv_length= to_cs->mbmaxlen * length / from_cs->mbminlen;
+  size_t conv_length= length * to_cs->mbmaxlen / from_cs->mbminlen;
   uint dummy_error;
   char *field_buf;
   if (!thd->mysql)            // bootstrap file handling
@@ -1404,10 +1414,10 @@ bool Protocol::net_store_data_cs(const uchar *from, size_t length,
   *next_field= field_buf + sizeof(uint);
   length= copy_and_convert(*next_field, conv_length, to_cs,
                            (const char*) from, length, from_cs, &dummy_error);
-  *(uint *) field_buf= length;
+  *(uint *) field_buf= (uint)length;
   (*next_field)[length]= 0;
   if (next_mysql_field->max_length < length)
-    next_mysql_field->max_length= length;
+    next_mysql_field->max_length= (ulong)length;
   ++next_field;
   ++next_mysql_field;
   return false;
