@@ -1,5 +1,5 @@
 /* Copyright (c) 2010, 2014, Oracle and/or its affiliates.
-   Copyright (c) 2013, 2020, MariaDB Corporation.
+   Copyright (c) 2013, 2021, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 class Alter_drop;
 class Alter_column;
 class Alter_rename_key;
+class Alter_index_ignorability;
 class Key;
 
 /**
@@ -96,6 +97,8 @@ public:
   List<Alter_rename_key>        alter_rename_key_list;
   // List of columns, used by both CREATE and ALTER TABLE.
   List<Create_field>            create_list;
+  // Indexes whose ignorability needs to be changed.
+  List<Alter_index_ignorability>  alter_index_ignorability_list;
   List<Virtual_column_info>     check_constraint_list;
   // Type of ALTER TABLE operation.
   alter_table_operations        flags;
@@ -130,6 +133,7 @@ public:
     key_list.empty();
     alter_rename_key_list.empty();
     create_list.empty();
+    alter_index_ignorability_list.empty();
     check_constraint_list.empty();
     flags= 0;
     partition_flags= 0;
@@ -249,13 +253,15 @@ public:
                   const LEX_CSTRING *new_db_arg, const LEX_CSTRING *new_name_arg);
 
   /**
-     @return true if the table is moved to another database, false otherwise.
+     @return true if the table is moved to another database or a new table
+     created by ALTER_PARTITION_CONVERT_OUT, false otherwise.
   */
   bool is_database_changed() const
   { return (new_db.str != db.str); };
 
   /**
-     @return true if the table is renamed, false otherwise.
+     @return true if the table is renamed or a new table created by
+     ALTER_PARTITION_CONVERT_OUT, false otherwise.
   */
   bool is_table_renamed() const
   { return (is_database_changed() || new_name.str != table_name.str); };
@@ -293,6 +299,12 @@ public:
   const char *get_tmp_path() const
   { return tmp_path; }
 
+  const LEX_CSTRING get_tmp_cstring_path() const
+  {
+    LEX_CSTRING tmp= { tmp_path, strlen(tmp_path) };
+    return tmp;
+  };
+
   /**
     Mark ALTER TABLE as needing to produce foreign key error if
     it deletes a row from the table being changed.
@@ -306,28 +318,38 @@ public:
 
   void report_implicit_default_value_error(THD *thd, const TABLE_SHARE *) const;
 public:
-  Create_field *implicit_default_value_error_field;
-  bool         error_if_not_empty;
-  uint         tables_opened;
+  Create_field *implicit_default_value_error_field= nullptr;
+  bool         error_if_not_empty= false;
+  uint         tables_opened= 0;
   LEX_CSTRING  db;
   LEX_CSTRING  table_name;
+  LEX_CSTRING  storage_engine_name;
   LEX_CSTRING  alias;
   LEX_CSTRING  new_db;
   LEX_CSTRING  new_name;
   LEX_CSTRING  new_alias;
   LEX_CSTRING  tmp_name;
+  LEX_CSTRING  tmp_storage_engine_name;
+  LEX_CUSTRING tmp_id, id;
   char         tmp_buff[80];
+  uchar        id_buff[MY_UUID_SIZE];
+  char         storage_engine_buff[NAME_LEN], tmp_storage_engine_buff[NAME_LEN];
+  bool         storage_engine_partitioned;
+  bool         tmp_storage_engine_name_partitioned;
+
   /**
     Indicates that if a row is deleted during copying of data from old version
     of table to the new version ER_FK_CANNOT_DELETE_PARENT error should be
     emitted.
   */
-  bool         fk_error_if_delete_row;
+  bool fk_error_if_delete_row= false;
   /** Name of foreign key for the above error. */
-  const char   *fk_error_id;
+  const char *fk_error_id= nullptr;
   /** Name of table for the above error. */
-  const char   *fk_error_table;
-  bool         modified_primary_key;
+  const char *fk_error_table= nullptr;
+  bool modified_primary_key= false;
+  /** Indicates that we are altering temporary table */
+  bool tmp_table= false;
 
 private:
   char new_filename[FN_REFLEN + 1];
@@ -337,11 +359,6 @@ private:
   char new_path[FN_REFLEN + 1];
   char tmp_path[FN_REFLEN + 1];
 
-#ifdef DBUG_ASSERT_EXISTS
-  /** Indicates that we are altering temporary table. Used only in asserts. */
-  bool tmp_table;
-#endif
-
   Alter_table_ctx &operator=(const Alter_table_ctx &rhs); // not implemented
   Alter_table_ctx(const Alter_table_ctx &rhs);            // not implemented
 };
@@ -362,7 +379,7 @@ protected:
 
   virtual ~Sql_cmd_common_alter_table() = default;
 
-  enum_sql_command sql_command_code() const override
+  virtual enum_sql_command sql_command_code() const
   {
     return SQLCOM_ALTER_TABLE;
   }
@@ -383,9 +400,9 @@ public:
 
   ~Sql_cmd_alter_table() = default;
 
-  Storage_engine_name *option_storage_engine_name() override { return this; }
+  Storage_engine_name *option_storage_engine_name() { return this; }
 
-  bool execute(THD *thd) override;
+  bool execute(THD *thd);
 };
 
 
@@ -405,11 +422,11 @@ public:
 
   ~Sql_cmd_alter_sequence() = default;
 
-  enum_sql_command sql_command_code() const override
+  enum_sql_command sql_command_code() const
   {
     return SQLCOM_ALTER_SEQUENCE;
   }
-  bool execute(THD *thd) override;
+  bool execute(THD *thd);
 };
 
 
@@ -429,7 +446,7 @@ public:
     : m_tablespace_op(tablespace_op_arg)
   {}
 
-  bool execute(THD *thd) override;
+  bool execute(THD *thd);
 
 private:
   const enum_tablespace_op_type m_tablespace_op;
