@@ -61,7 +61,6 @@ Created 10/8/1995 Heikki Tuuri
 #include "srv0start.h"
 #include "trx0i_s.h"
 #include "trx0purge.h"
-#include "ut0crc32.h"
 #include "btr0defragment.h"
 #include "ut0mem.h"
 #include "fil0fil.h"
@@ -97,15 +96,18 @@ char*	srv_data_home;
 char*	srv_undo_dir;
 
 /** The number of tablespaces to use for rollback segments. */
-ulong	srv_undo_tablespaces;
+uint	srv_undo_tablespaces;
 
 /** The number of UNDO tablespaces that are open and ready to use. */
-ulint	srv_undo_tablespaces_open;
+uint32_t srv_undo_tablespaces_open;
 
 /** The number of UNDO tablespaces that are active (hosting some rollback
 segment). It is quite possible that some of the tablespaces doesn't host
 any of the rollback-segment based on configuration used. */
-ulint	srv_undo_tablespaces_active;
+uint32_t srv_undo_tablespaces_active;
+
+/** Rate at which UNDO records should be purged. */
+ulong	srv_purge_rseg_truncate_frequency;
 
 /** Enable or Disable Truncate of UNDO tablespace.
 Note: If enabled then UNDO tablespace will be selected for truncate.
@@ -150,8 +152,6 @@ char*	srv_log_group_home_dir;
 /** The InnoDB redo log file size, or 0 when changing the redo log format
 at startup (while disallowing writes to the redo log). */
 ulonglong	srv_log_file_size;
-/** innodb_log_buffer_size, in bytes */
-ulong		srv_log_buffer_size;
 /** innodb_flush_log_at_trx_commit */
 ulong		srv_flush_log_at_trx_commit;
 /** innodb_flush_log_at_timeout */
@@ -159,9 +159,7 @@ uint		srv_flush_log_at_timeout;
 /** innodb_page_size */
 ulong		srv_page_size;
 /** log2 of innodb_page_size; @see innodb_init_params() */
-ulong		srv_page_size_shift;
-/** innodb_log_write_ahead_size */
-ulong		srv_log_write_ahead_size;
+uint32_t	srv_page_size_shift;
 
 /** innodb_adaptive_flushing; try to flush dirty pages so as to avoid
 IO bursts at the checkpoints. */
@@ -185,9 +183,12 @@ with mysql_mutex_lock(), which will wait until it gets the mutex. */
 
 /** copy of innodb_buffer_pool_size */
 ulint	srv_buf_pool_size;
-/** Requested buffer pool chunk size. Each buffer pool instance consists
-of one or more chunks. */
-ulong	srv_buf_pool_chunk_unit;
+/** Requested buffer pool chunk size */
+size_t	srv_buf_pool_chunk_unit;
+/** innodb_lru_scan_depth; number of blocks scanned in LRU flush batch */
+ulong	srv_LRU_scan_depth;
+/** innodb_flush_neighbors; whether or not to flush neighbors of a block */
+ulong	srv_flush_neighbors;
 /** Previously requested size */
 ulint	srv_buf_pool_old_size;
 /** Current size as scaling factor for the other components */
@@ -499,8 +500,26 @@ static srv_sys_t	srv_sys;
 struct purge_coordinator_state
 {
   /** Snapshot of the last history length before the purge call.*/
-  size_t history_size;
+  size_t m_history_length;
   Atomic_counter<int> m_running;
+private:
+  ulint count;
+  ulint n_use_threads;
+  ulint n_threads;
+
+  ulint lsn_lwm;
+  ulint lsn_hwm;
+  ulonglong start_time;
+  ulint lsn_age_factor;
+
+  static constexpr ulint adaptive_purge_threshold= 20;
+  static constexpr ulint safety_net= 20;
+  ulint series[innodb_purge_threads_MAX + 1];
+
+  inline void compute_series();
+  inline void lazy_init();
+  void refresh(bool full);
+
 public:
   inline void do_purge();
 };
@@ -678,8 +697,6 @@ static void srv_refresh_innodb_monitor_stats(time_t current_time)
 	btr_cur_n_non_sea_old = btr_cur_n_non_sea;
 #endif /* BTR_CUR_HASH_ADAPT */
 
-	log_refresh_stats();
-
 	buf_refresh_io_stats();
 
 	srv_n_rows_inserted_old = srv_stats.n_rows_inserted;
@@ -965,12 +982,6 @@ srv_export_innodb_status(void)
 	export_vars.innodb_data_pending_writes =
 		ulint(MONITOR_VALUE(MONITOR_OS_PENDING_WRITES));
 
-	export_vars.innodb_data_pending_fsyncs =
-		log_sys.get_pending_flushes()
-		+ fil_n_pending_tablespace_flushes;
-
-	export_vars.innodb_data_fsyncs = os_n_fsyncs;
-
 	export_vars.innodb_data_read = srv_stats.data_read;
 
 	export_vars.innodb_data_reads = os_n_file_reads;
@@ -986,12 +997,6 @@ srv_export_innodb_status(void)
 	export_vars.innodb_data_written = srv_stats.data_written
 		+ (dblwr << srv_page_size_shift);
 
-	export_vars.innodb_buffer_pool_read_requests
-		= buf_pool.stat.n_page_gets;
-
-	export_vars.innodb_buffer_pool_write_requests =
-		srv_stats.buf_pool_write_requests;
-
 	export_vars.innodb_buffer_pool_bytes_data =
 		buf_pool.stat.LRU_bytes
 		+ (UT_LIST_GET_LEN(buf_pool.unzip_LRU)
@@ -1011,22 +1016,6 @@ srv_export_innodb_status(void)
 	export_vars.innodb_max_trx_id = trx_sys.get_max_trx_id();
 	export_vars.innodb_history_list_length = trx_sys.history_size_approx();
 
-	export_vars.innodb_log_waits = srv_stats.log_waits;
-
-	export_vars.innodb_os_log_written = srv_stats.os_log_written;
-
-	export_vars.innodb_os_log_fsyncs = log_sys.get_flushes();
-
-	export_vars.innodb_os_log_pending_fsyncs
-		= log_sys.get_pending_flushes();
-
-	export_vars.innodb_os_log_pending_writes =
-		srv_stats.os_log_pending_writes;
-
-	export_vars.innodb_log_write_requests = srv_stats.log_write_requests;
-
-	export_vars.innodb_log_writes = srv_stats.log_writes;
-
 	mysql_mutex_lock(&lock_sys.wait_mutex);
 	export_vars.innodb_row_lock_waits = lock_sys.get_wait_cumulative();
 
@@ -1113,13 +1102,15 @@ srv_export_innodb_status(void)
 
 	mysql_mutex_unlock(&srv_innodb_monitor_mutex);
 
-	mysql_mutex_lock(&log_sys.mutex);
+	log_sys.latch.rd_lock(SRW_LOCK_CALL);
 	export_vars.innodb_lsn_current = log_sys.get_lsn();
 	export_vars.innodb_lsn_flushed = log_sys.get_flushed_lsn();
 	export_vars.innodb_lsn_last_checkpoint = log_sys.last_checkpoint_lsn;
 	export_vars.innodb_checkpoint_max_age = static_cast<ulint>(
 		log_sys.max_checkpoint_age);
-	mysql_mutex_unlock(&log_sys.mutex);
+	log_sys.latch.rd_unlock();
+	export_vars.innodb_os_log_written = export_vars.innodb_lsn_current
+		- recv_sys.lsn;
 
 	export_vars.innodb_checkpoint_age = static_cast<ulint>(
 		export_vars.innodb_lsn_current
@@ -1197,7 +1188,7 @@ static void srv_monitor()
 void srv_monitor_task(void*)
 {
 	/* number of successive fatal timeouts observed */
-	static lsn_t		old_lsn = recv_sys.recovered_lsn;
+	static lsn_t		old_lsn = recv_sys.lsn;
 
 	ut_ad(!srv_read_only_mode);
 
@@ -1220,7 +1211,6 @@ void srv_monitor_task(void*)
 			now -= start;
 			ulong waited = static_cast<ulong>(now / 1000000);
 			if (waited >= threshold) {
-				buf_pool.print_flush_info();
 				ib::fatal() << dict_sys.fatal_msg;
 			}
 
@@ -1260,13 +1250,6 @@ bool srv_any_background_activity()
 
 static void purge_worker_callback(void*);
 static void purge_coordinator_callback(void*);
-static void purge_truncation_callback(void*)
-{
-  purge_sys.latch.rd_lock(SRW_LOCK_CALL);
-  const purge_sys_t::iterator head= purge_sys.head;
-  purge_sys.latch.rd_unlock();
-  head.free_history();
-}
 
 static tpool::task_group purge_task_group;
 tpool::waitable_task purge_worker_task(purge_worker_callback, nullptr,
@@ -1274,47 +1257,57 @@ tpool::waitable_task purge_worker_task(purge_worker_callback, nullptr,
 static tpool::task_group purge_coordinator_task_group(1);
 static tpool::waitable_task purge_coordinator_task
   (purge_coordinator_callback, nullptr, &purge_coordinator_task_group);
-static tpool::task_group purge_truncation_task_group(1);
-static tpool::waitable_task purge_truncation_task
-  (purge_truncation_callback, nullptr, &purge_truncation_task_group);
+
+static tpool::timer *purge_coordinator_timer;
 
 /** Wake up the purge threads if there is work to do. */
-void purge_sys_t::wake_if_not_active()
+void srv_wake_purge_thread_if_not_active()
 {
-  if (enabled() && !paused() && !purge_state.m_running &&
+  ut_ad(!srv_read_only_mode);
+
+  if (purge_sys.enabled() && !purge_sys.paused() &&
       (srv_undo_log_truncate || trx_sys.history_exists()) &&
       ++purge_state.m_running == 1)
     srv_thread_pool->submit_task(&purge_coordinator_task);
 }
 
 /** @return whether the purge tasks are active */
-bool purge_sys_t::running()
+bool purge_sys_t::running() const
 {
   return purge_coordinator_task.is_running();
 }
 
-void purge_sys_t::stop_FTS()
+/** Suspend purge in data dictionary tables */
+void purge_sys_t::stop_SYS()
 {
-  ut_d(const auto paused=) m_FTS_paused.fetch_add(1);
-  ut_ad((paused + 1) & ~PAUSED_SYS);
-  while (m_active.load(std::memory_order_acquire))
-    std::this_thread::sleep_for(std::chrono::seconds(1));
+  latch.rd_lock(SRW_LOCK_CALL);
+  ++m_SYS_paused;
+  latch.rd_unlock();
 }
 
 /** Stop purge during FLUSH TABLES FOR EXPORT */
 void purge_sys_t::stop()
 {
-  latch.wr_lock(SRW_LOCK_CALL);
-
-  if (!enabled())
+  for (;;)
   {
-    /* Shutdown must have been initiated during FLUSH TABLES FOR EXPORT. */
-    ut_ad(!srv_undo_sources);
-    latch.wr_unlock();
-    return;
-  }
+    latch.wr_lock(SRW_LOCK_CALL);
 
-  ut_ad(srv_n_purge_threads > 0);
+    if (!enabled())
+    {
+      /* Shutdown must have been initiated during FLUSH TABLES FOR EXPORT. */
+      ut_ad(!srv_undo_sources);
+      latch.wr_unlock();
+      return;
+    }
+
+    ut_ad(srv_n_purge_threads > 0);
+
+    if (!must_wait_SYS())
+      break;
+
+    latch.wr_unlock();
+    std::this_thread::sleep_for(std::chrono::seconds(1));
+  }
 
   const auto paused= m_paused++;
 
@@ -1331,8 +1324,9 @@ void purge_sys_t::stop()
 /** Resume purge in data dictionary tables */
 void purge_sys_t::resume_SYS(void *)
 {
-  ut_d(auto paused=) purge_sys.m_FTS_paused.fetch_sub(PAUSED_SYS);
-  ut_ad(paused >= PAUSED_SYS);
+  ut_d(const auto s=)
+  purge_sys.m_SYS_paused--;
+  ut_ad(s);
 }
 
 /** Resume purge at UNLOCK TABLES after FLUSH TABLES FOR EXPORT */
@@ -1354,8 +1348,8 @@ void purge_sys_t::resume()
    if (paused == 1)
    {
      ib::info() << "Resuming purge";
-     purge_state.m_running= 1;
-     srv_thread_pool->submit_task(&purge_coordinator_task);
+     purge_state.m_running = 0;
+     srv_wake_purge_thread_if_not_active();
      MONITOR_ATOMIC_INC(MONITOR_PURGE_RESUME_COUNT);
    }
    latch.wr_unlock();
@@ -1500,7 +1494,8 @@ void srv_master_callback(void*)
   ut_a(srv_shutdown_state <= SRV_SHUTDOWN_INITIATED);
 
   MONITOR_INC(MONITOR_MASTER_THREAD_SLEEP);
-  purge_sys.wake_if_not_active();
+  if (!purge_state.m_running)
+    srv_wake_purge_thread_if_not_active();
   ulonglong counter_time= microsecond_interval_timer();
   srv_sync_log_buffer_in_background();
   MONITOR_INC_TIME_IN_MICRO_SECS(MONITOR_SRV_LOG_FLUSH_MICROSECOND,
@@ -1553,6 +1548,7 @@ static bool srv_purge_should_exit(size_t old_history_size)
 
 /*********************************************************************//**
 Fetch and execute a task from the work queue.
+@param [in,out]	slot	purge worker thread slot
 @return true if a task was executed */
 static bool srv_task_execute()
 {
@@ -1576,80 +1572,202 @@ static bool srv_task_execute()
 
 static void purge_create_background_thds(int );
 
-/** Flag which is set, whenever innodb_purge_threads changes. */
-static Atomic_relaxed<bool> srv_purge_thread_count_changed;
-
-static std::mutex purge_thread_count_mtx;
+std::mutex purge_thread_count_mtx;
 void srv_update_purge_thread_count(uint n)
 {
 	std::lock_guard<std::mutex> lk(purge_thread_count_mtx);
-	purge_create_background_thds(n);
+	ut_ad(n > 0);
+	ut_ad(n <= innodb_purge_threads_MAX);
 	srv_n_purge_threads = n;
-	srv_purge_thread_count_changed = true;
+	srv_purge_thread_count_changed = 1;
 }
 
+Atomic_counter<int> srv_purge_thread_count_changed;
+
 inline void purge_coordinator_state::do_purge()
 {
   ut_ad(!srv_read_only_mode);
+  lazy_init();
+  ut_ad(n_threads);
+  bool wakeup= false;
 
-  if (!purge_sys.enabled() || purge_sys.paused())
-    return;
-
-  uint n_threads;
+  purge_coordinator_timer->disarm();
 
+  while (purge_sys.enabled() && !purge_sys.paused())
   {
-    std::lock_guard<std::mutex> lk(purge_thread_count_mtx);
-    n_threads= srv_n_purge_threads;
-    srv_purge_thread_count_changed= false;
-    goto first_loop;
-  }
+loop:
+    wakeup= false;
+    const auto now= my_interval_timer();
+    const auto sigcount= m_running;
+
+    if (now - start_time >= 1000000)
+    {
+      refresh(false);
+      start_time= now;
+    }
+
+    const auto old_activity_count= srv_sys.activity_count;
+    const auto history_size= trx_sys.history_size();
 
-  do
-  {
     if (UNIV_UNLIKELY(srv_purge_thread_count_changed))
     {
       /* Read the fresh value of srv_n_purge_threads, reset
-      the changed flag. Both are protected by purge_thread_count_mtx. */
+      the changed flag. Both are protected by purge_thread_count_mtx.
+
+      This code does not run concurrently, it is executed
+      by a single purge_coordinator thread, and no races
+      involving srv_purge_thread_count_changed are possible. */
       {
         std::lock_guard<std::mutex> lk(purge_thread_count_mtx);
-        n_threads= srv_n_purge_threads;
-        srv_purge_thread_count_changed= false;
+        n_threads= n_use_threads= srv_n_purge_threads;
+        srv_purge_thread_count_changed= 0;
+      }
+      refresh(true);
+      start_time= now;
+    }
+    else if (history_size > m_history_length)
+    {
+      /* dynamically adjust the purge thread based on redo log fill factor */
+      if (n_use_threads < n_threads && lsn_age_factor < lsn_lwm)
+      {
+more_threads:
+        ++n_use_threads;
+        lsn_hwm= lsn_lwm;
+        lsn_lwm-= series[n_use_threads];
+      }
+      else if (n_use_threads > 1 && lsn_age_factor >= lsn_hwm)
+      {
+fewer_threads:
+        --n_use_threads;
+        lsn_lwm= lsn_hwm;
+        lsn_hwm+= series[n_use_threads];
+      }
+      else if (n_use_threads == 1 && lsn_age_factor >= 100 - safety_net)
+      {
+        wakeup= true;
+        break;
       }
     }
-  first_loop:
-    ut_ad(n_threads);
+    else if (n_threads > n_use_threads &&
+             srv_max_purge_lag && m_history_length > srv_max_purge_lag)
+      goto more_threads;
+    else if (n_use_threads > 1 && old_activity_count == srv_sys.activity_count)
+      goto fewer_threads;
 
-    history_size= trx_sys.history_size();
+    ut_ad(n_use_threads);
+    ut_ad(n_use_threads <= n_threads);
+
+    m_history_length= history_size;
 
     if (!history_size)
     {
-    no_history:
       srv_dml_needed_delay= 0;
-      purge_truncation_task.wait();
-      trx_purge_truncate_history();
-      break;
-    }
-
-    ulint n_pages_handled= trx_purge(n_threads, history_size);
-    if (!trx_sys.history_exists())
-      goto no_history;
-    if (purge_sys.truncating_tablespace() ||
-        srv_shutdown_state != SRV_SHUTDOWN_NONE)
-    {
-      purge_truncation_task.wait();
       trx_purge_truncate_history();
     }
     else
-      srv_thread_pool->submit_task(&purge_truncation_task);
-    if (!n_pages_handled)
+    {
+      ulint n_pages_handled= trx_purge(n_use_threads, history_size);
+      if (!(++count % srv_purge_rseg_truncate_frequency) ||
+          purge_sys.truncate.current ||
+          (srv_shutdown_state != SRV_SHUTDOWN_NONE && srv_fast_shutdown == 0))
+        trx_purge_truncate_history();
+      if (n_pages_handled)
+        continue;
+    }
+
+    if (srv_dml_needed_delay);
+    else if (m_running == sigcount)
+    {
+      /* Purge was not woken up by srv_wake_purge_thread_if_not_active() */
+
+      /* The magic number 5000 is an approximation for the case where we have
+      cached undo log records which prevent truncate of rollback segments. */
+      wakeup= history_size >= 5000 ||
+        (history_size && history_size != trx_sys.history_size_approx());
       break;
+    }
+
+    if (!trx_sys.history_exists())
+    {
+      srv_dml_needed_delay= 0;
+      break;
+    }
+
+    if (!srv_purge_should_exit(history_size))
+      goto loop;
   }
-  while (purge_sys.enabled() && !purge_sys.paused() &&
-         !srv_purge_should_exit(history_size));
+
+  if (wakeup)
+    purge_coordinator_timer->set_time(10, 0);
 
   m_running= 0;
 }
 
+inline void purge_coordinator_state::compute_series()
+{
+  ulint points= n_threads;
+  memset(series, 0, sizeof series);
+  constexpr ulint spread= 100 - adaptive_purge_threshold - safety_net;
+
+  /* We distribute spread across n_threads,
+  e.g.: spread of 60 is distributed across n_threads=4 as: 6+12+18+24 */
+
+  const ulint additional_points= (points * (points + 1)) / 2;
+  if (spread % additional_points == 0)
+  {
+    /* Arithmetic progression is possible. */
+    const ulint delta= spread / additional_points;
+    ulint growth= delta;
+    do
+    {
+      series[points--]= growth;
+      growth += delta;
+    }
+    while (points);
+    return;
+  }
+
+  /* Use average distribution to spread across the points */
+  const ulint delta= spread / points;
+  ulint total= 0;
+  do
+  {
+    series[points--]= delta;
+    total+= delta;
+  }
+  while (points);
+
+  for (points= 1; points <= n_threads && total++ < spread; )
+    series[points++]++;
+}
+
+inline void purge_coordinator_state::lazy_init()
+{
+  if (n_threads)
+    return;
+  n_threads= n_use_threads= srv_n_purge_threads;
+  refresh(true);
+  start_time= my_interval_timer();
+}
+
+void purge_coordinator_state::refresh(bool full)
+{
+  if (full)
+  {
+    compute_series();
+    lsn_lwm= adaptive_purge_threshold;
+    lsn_hwm= adaptive_purge_threshold + series[n_threads];
+  }
+
+  log_sys.latch.rd_lock(SRW_LOCK_CALL);
+  const lsn_t last= log_sys.last_checkpoint_lsn,
+    max_age= log_sys.max_checkpoint_age;
+  log_sys.latch.rd_unlock();
+
+  lsn_age_factor= ulint(((log_sys.get_lsn() - last) * 100) / max_age);
+}
+
+
 static std::list<THD*> purge_thds;
 static std::mutex purge_thd_mutex;
 extern void* thd_attach_thd(THD*);
@@ -1692,13 +1810,6 @@ static void release_thd(THD *thd, void *ctx)
 	set_current_thd(0);
 }
 
-void srv_purge_worker_task_low()
-{
-  ut_ad(current_thd);
-  while (srv_task_execute())
-    ut_ad(purge_sys.running());
-}
-
 static void purge_worker_callback(void*)
 {
   ut_ad(!current_thd);
@@ -1706,7 +1817,8 @@ static void purge_worker_callback(void*)
   ut_ad(srv_force_recovery < SRV_FORCE_NO_BACKGROUND);
   void *ctx;
   THD *thd= acquire_thd(&ctx);
-  srv_purge_worker_task_low();
+  while (srv_task_execute())
+    ut_ad(purge_sys.running());
   release_thd(thd,ctx);
 }
 
@@ -1720,13 +1832,16 @@ static void purge_coordinator_callback(void*)
 
 void srv_init_purge_tasks()
 {
-  purge_create_background_thds(srv_n_purge_threads);
-  purge_sys.coordinator_startup();
+  purge_create_background_thds(innodb_purge_threads_MAX);
+  purge_coordinator_timer= srv_thread_pool->create_timer
+    (purge_coordinator_callback, nullptr);
 }
 
 static void srv_shutdown_purge_tasks()
 {
   purge_coordinator_task.disable();
+  delete purge_coordinator_timer;
+  purge_coordinator_timer= nullptr;
   purge_worker_task.wait();
   std::unique_lock<std::mutex> lk(purge_thd_mutex);
   while (!purge_thds.empty())
@@ -1735,7 +1850,6 @@ static void srv_shutdown_purge_tasks()
     purge_thds.pop_front();
   }
   n_purge_thds= 0;
-  purge_truncation_task.wait();
 }
 
 /**********************************************************************//**
@@ -1778,16 +1892,13 @@ void srv_purge_shutdown()
   if (purge_sys.enabled())
   {
     if (!srv_fast_shutdown && !opt_bootstrap)
-    {
-      srv_purge_batch_size= innodb_purge_batch_size_MAX;
       srv_update_purge_thread_count(innodb_purge_threads_MAX);
-    }
     size_t history_size= trx_sys.history_size();
     while (!srv_purge_should_exit(history_size))
     {
       history_size= trx_sys.history_size();
       ut_a(!purge_sys.paused());
-      srv_thread_pool->submit_task(&purge_coordinator_task);
+      srv_wake_purge_thread_if_not_active();
       purge_coordinator_task.wait();
     }
     purge_sys.coordinator_shutdown();
