@@ -126,6 +126,25 @@ public:
     bool use_fields,
     spider_fields *fields
   ) override;
+protected:
+  int check_item_func(
+    Item_func *item_func,
+    ha_spider *spider,
+    const char *alias,
+    uint alias_length,
+    bool use_fields,
+    spider_fields *fields
+  );
+  int print_item_func(
+    Item_func *item_func,
+    ha_spider *spider,
+    spider_string *str,
+    const char *alias,
+    uint alias_length,
+    bool use_fields,
+    spider_fields *fields
+  );
+public:
   int open_item_sum_func(
     Item_sum *item_sum,
     ha_spider *spider,
@@ -191,13 +210,13 @@ public:
   int append_having(
     spider_string *str
   ) override;
-  bool tables_on_different_db_are_joinable();
-  bool socket_has_default_value();
-  bool database_has_default_value();
-  bool default_file_has_default_value();
-  bool host_has_default_value();
-  bool port_has_default_value();
-  bool append_charset_name_before_string();
+  bool tables_on_different_db_are_joinable() override;
+  bool socket_has_default_value() override;
+  bool database_has_default_value() override;
+  bool default_file_has_default_value() override;
+  bool host_has_default_value() override;
+  bool port_has_default_value() override;
+  bool append_charset_name_before_string() override;
 };
 
 class spider_db_mysql_util: public spider_db_mbase_util
@@ -440,9 +459,7 @@ public:
   bool is_xa_nota_error(
     int error_num
   );
-  int print_warnings(
-    struct tm *l_time
-  );
+  int fetch_and_print_warnings(struct tm *l_time);
   spider_db_result *store_result(
     spider_db_result_buffer **spider_res_buf,
     st_spider_db_request_key *request_key,
