@@ -22,8 +22,10 @@
 
 #include "strings_def.h"
 #include <m_ctype.h>
+#include "ctype-simple.h"
 
 const char charset_name_binary[]= "binary";
+#define charset_name_binary_length (sizeof(charset_name_binary)-1)
 
 static const uchar ctype_bin[]=
 {
@@ -84,7 +86,7 @@ static int my_strnncoll_binary(CHARSET_INFO * cs __attribute__((unused)),
                                my_bool t_is_prefix)
 {
   size_t len=MY_MIN(slen,tlen);
-  int cmp= memcmp(s,t,len);
+  int cmp= len ? memcmp(s, t, len) : 0;
   return cmp ? cmp : (int)((t_is_prefix ? len : slen) - tlen);
 }
 
@@ -145,7 +147,7 @@ static int my_strnncoll_8bit_bin(CHARSET_INFO * cs __attribute__((unused)),
                                  my_bool t_is_prefix)
 {
   size_t len=MY_MIN(slen,tlen);
-  int cmp= memcmp(s,t,len);
+  int cmp= len ? memcmp(s, t, len) : 0;
   return cmp ? cmp : (int)((t_is_prefix ? len : slen) - tlen);
 }
 
@@ -521,7 +523,9 @@ MY_COLLATION_HANDLER my_collation_8bit_bin_handler =
   my_strcasecmp_bin,
   my_instr_bin,
   my_hash_sort_8bit_bin,
-  my_propagate_simple
+  my_propagate_simple,
+  my_min_str_8bit_simple,
+  my_max_str_8bit_simple
 };
 
 
@@ -538,7 +542,9 @@ MY_COLLATION_HANDLER my_collation_8bit_nopad_bin_handler =
   my_strcasecmp_bin,
   my_instr_bin,
   my_hash_sort_bin,
-  my_propagate_simple
+  my_propagate_simple,
+  my_min_str_8bit_simple_nopad,
+  my_max_str_8bit_simple
 };
 
 
@@ -555,7 +561,9 @@ static MY_COLLATION_HANDLER my_collation_binary_handler =
   my_strcasecmp_bin,
   my_instr_bin,
   my_hash_sort_bin,
-  my_propagate_simple
+  my_propagate_simple,
+  my_min_str_8bit_simple_nopad,
+  my_max_str_8bit_simple
 };
 
 
@@ -597,8 +605,8 @@ struct charset_info_st my_charset_bin =
 {
     63,0,0,			/* number        */
     MY_CS_COMPILED|MY_CS_BINSORT|MY_CS_PRIMARY|MY_CS_NOPAD,/* state */
-    charset_name_binary,	/* cs name    */
-    "binary",			/* name          */
+    { charset_name_binary, charset_name_binary_length},	/* cs name    */
+    { STRING_WITH_LEN("binary")},	                /* name          */
     "",				/* comment       */
     NULL,			/* tailoring     */
     ctype_bin,			/* ctype         */
@@ -624,3 +632,7 @@ struct charset_info_st my_charset_bin =
     &my_charset_handler,
     &my_collation_binary_handler
 };
+
+
+struct charset_info_st my_collation_contextually_typed_binary= {0};
+struct charset_info_st my_collation_contextually_typed_default= {0};
